# ------------------------------------------------------------------------------------------------
# Software:      Phondler
#
# Description:   Python handler for phonetic information
# Version:       0.0.1
# Module:        Main
#
# Authors:       Frederic Bayer, Mark Hasegawa-Johnson, Ruoho Ruotsi
# Email:         frederic.s.bayer@gmail.com
#
# Copyright:     (c) Frederic Bayer & Ruoho Ruotsi 2021, Mark Hasegawa-Johnson 2018-2021
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the MIT license.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# ------------------------------------------------------------------------------------------------
__authors__ = "Frederic Bayer, Mark Hasegawa-Johnson, Ruoho Ruotsi"
__version__ = "0.0.1"
__date__ = "2021"

import phondler.phondler